#include <ostream>
#include <unistd.h>
#include "MCFgen.hpp"  // tostring, die
#include "args.hpp"

namespace args {
  // definitions & defaults:
  string seqfile, matfile, mat_title, outfile;
  double score_thresh = 5;  // ??
  uint bg_range = 100;
  bool even_bg = false;
  bool mask_lower = false;
  double pseudo = 0.375;
}

void args::parse(int argc, char ** argv)
{
  const string usage =
    "Usage: possum [options] matrixfile myseqs.fa\n"
    "Options:\n"
    "-r  restrict search to cis-element with this title\n"
    "-t  score threshold for reporting cis-elements (" + mcf::tostring(score_thresh) + ")\n"
    "-o  write output to this file (default = stdout)\n"
    "-w  range in bp for counting local nucleotide abundances (" + mcf::tostring(bg_range) + ")\n"
    "-q  pretend residue abundances = 1/4\n"
    "-l  filter lowercase letters in sequences\n"
    "-p  pseudocount to add to all matrix elements (" + mcf::tostring(pseudo) + ")\n"
    ;

  int c;

  while ((c = getopt(argc, argv, "r:t:o:w:qlp:")) != -1)
    switch (c) {
    case 'r':
      mat_title = optarg;
      break;
    case 't':
      score_thresh = atof(optarg);
      break;
    case 'o':
      outfile = optarg;
      break;
    case 'w':
      bg_range = atoi(optarg);  // what happens if user enters -ve number?
      break;
    case 'q':
      even_bg = true;
      break;
    case 'l':
      mask_lower = true;
      break;
    case 'p':
      pseudo = atof(optarg);
      if (pseudo < 0) mcf::die("Pseudocount should be >= zero");
      break;
    case '?':
      mcf::die(usage);
    }

  if (optind >= argc)  // there should be at least 1 non-option argument
    mcf::die(usage);

  matfile = argv[optind++];
  if (optind < argc)
    seqfile = argv[optind++];
}

void args::print(ostream & strm)
{
  strm
    << "Sequence file: " << seqfile << '\n'
    << "Matrix file: " << matfile << '\n'
    << "Cis-element: " << (mat_title.empty() ? "(all)" : mat_title) << '\n'
    << "Score threshold: " << score_thresh << '\n'
    ;
  if (even_bg)
    strm << "Using residue abundances = 1/4\n";
  else
    strm << "Range for local abundances: " << bg_range << '\n';
  strm
    << "Lowercase filtering: " << (mask_lower ? "ON\n" : "OFF\n")
    << "Pseudocount: " << pseudo << '\n'
    << flush;
}
