#include <string>
#include <vector>

using namespace std;

struct cis_element_submodel {
  string label;
  double plus_strand_weight;
  double minus_strand_weight;
  vector<int> transparency_mask;  // 1 = transparent, 0 = opaque
  vector<vector<double> > matrix;  // holds either counts or probabilities

  // constructors:

  cis_element_submodel() {}  // need this??

  cis_element_submodel(const string & l, double p, double m, const vector<int> & t, const vector<vector<double> > & mat)
    : label(l), plus_strand_weight(p), minus_strand_weight(m), transparency_mask(t), matrix(mat) {}

  void set_undefined_weights(double default_weight);
  void counts2probs(double pseudocount);
};

struct cis_element_model {
  string name;
  vector<cis_element_submodel> submodels;

  void set_undefined_weights(void);  // set undefined weights to 1 / (number of submodels)
  void normalize_weights_within(void);  // rescale weights so their sum = 1
  void counts2probs(double pseudocount);  // convert counts to probabilities

  // constructors:
  cis_element_model() {}

  cis_element_model(const string & n, const vector<cis_element_submodel> & s)
    : name(n), submodels(s) {}

  // read all cis-elements from the file
  static void read_file(const string & filename, vector<cis_element_model> & models);

  // just get the cis-element with the specified title
  static void read_one(const string & filename, const string & title, cis_element_model & model);
};
